
;*********************************************************************************
;* flow.s:
;*	Statement flow.
;*********************************************************************************

;********************************************************************************
;* This file is part of gibl:							*
;*	Gordons Interactive Basic Language                                      *
;********************************************************************************
;*    A Tiny Basic for the 6502 inspired by the NIBL Tiny Basic interpreter;	*
;*    "National Industrial Basic Language" originally for the INS8060 SC/MP	*
;*    system.									*
;*										*
;*    gibl is distributed under a "Source Available" license.			*
;*	It is NOT Open source and must not be treated as such.			*
;*										*
;*    See the file LICENSE for details.						*
;*										*
;*    gibl is Copyright (c) 2023 by Gordon Henderson				*
;********************************************************************************

	.macpack	generic

	.include	"data.h"
	.include	"error.h"

	.include	"il.h"
	.include	"ilMacros.h"
	.include	"ilUtils.h"

	.include	"flow.h"



;********************************************************************************
;* stmtDone:
;*	Check current statement finished.
;*	Moves the cursor over any trailing spaces and stops at a colon or CR.
;*	Error if no colon or CR.
;********************************************************************************

.proc	stmtDone
	skipSpaces
	cmp	#$0D			; CR?
	beq	doneCR
	cmp	#':'			; Colon?
	beq	doneCLN

	lda	#eCHAR
	jmp	ERR1


doneCLN:
	incCursor				; Skip over the colon
doneCR:
	rts				; ... but we leave the cursor on the CR for NXT
.endproc


;********************************************************************************
;* stmtNext:
;*	Next statement
;*	(Not FOR/NEXT)
;********************************************************************************

.proc	stmtNext
	lda	runMod
	beq	progFin


; See if we've landed on a CR

	ldy	#0
	lda	(cursor),y
	cmp	#$0D
	bne	:+

	incCursor		; Skip over it
	lda	(cursor),y	; Get next byte from the program
	bmi	progFin		; Top bit set - end

; Store in the line number

	sta	hiLine
	incCursor
	lda	(cursor),y
	sta	loLine
	incCursor
	incCursor			; Skip over the line length byte

:	lda	#>ilStatement	; Point I.L. to the next statement handler
	sta	pcH
	lda	#<ilStatement
	sta	pcL

	rts
.endproc


;********************************************************************************
;* progFin:
;*	Finish execution
;********************************************************************************

.proc	progFin
	lda	#0
	sta	runMod		; Not running
	sta	arithPtr	; Reset arithmetic stack
	sta	pcPtr		; and I.L. call stack

	lda	#<ilStart	; Set I.L. PC to ilStart
	sta	pcL
	lda	#>ilStart
	sta	pcH
	rts
.endproc


;********************************************************************************
;* progStart:
;*	Start execution
;********************************************************************************

.proc	progStart
	lda	#1
	sta	runMod		; Run mode = 1

	jsr	resetCursor

	lda	#0
	sta	sbrPtr		; Clear the stacks
	sta	forPtr
	sta	doPtr

; Store first line number
;	Remember, it's stored high byte first as the top-bit set indicates
;	end of program text.

	tay
	lda	(cursor),y
	sta	hiLine
	incCursor
	lda	(cursor),y
	sta	loLine
	incCursor
	incCursor			; Skip over line length

	rts
.endproc
