
;*********************************************************************************
;* logic.s:
;*********************************************************************************

;********************************************************************************
;* This file is part of gibl:							*
;*	Gordons Interactive Basic Language                                      *
;********************************************************************************
;*    A Tiny Basic for the 6502 inspired by the NIBL Tiny Basic interpreter;	*
;*    "National Industrial Basic Language" originally for the INS8060 SC/MP	*
;*    system.									*
;*										*
;*    gibl is distributed under a "Source Available" license.			*
;*	It is NOT Open source and must not be treated as such.			*
;*										*
;*    See the file LICENSE for details.						*
;*										*
;*    gibl is Copyright (c) 2023 by Gordon Henderson				*
;********************************************************************************

	.macpack	generic

	.include	"data.h"

	.include	"logic.h"


;********************************************************************************
;* Logical operators.
;*	AND, OR and NOT
;********************************************************************************


;********************************************************************************
;* AND:
;*	Also has all the return shenanigans.
;********************************************************************************

.proc	andOP
	jsr	pull
	lda	p2L
	and	p3L
	sta	p3L
	lda	p2H
	and	p3H
	sta	p3H			; Fall into ...
.endproc


;********************************************************************************
;* push:
;*	Push 2 words into the stack
;********************************************************************************

push:
	lda	p3L
	sta	arithStack-4,x		; Push low byte first
	lda	p3H
	sta	arithStack-3,x

	dex				; Fix stack pointer
	dex
	stx	arithPtr

	rts


;********************************************************************************
;* pull:
;*	Pull the 2 words off the stack
;********************************************************************************

.proc	pull

; Pull numbers off the stack, store one in p2, the other in p3
;	Note that we leave the stack pointer in X for later...

	ldx	arithPtr
	lda	arithStack-1,x
	sta	p2H
	lda	arithStack-2,x
	sta	p2L
	lda	arithStack-3,x
	sta	p3H
	lda	arithStack-4,x
	sta	p3L
	rts
.endproc


;********************************************************************************
;* OR:
;********************************************************************************

.proc	orOP
	jsr	pull
	lda	p2L
	ora	p3L
	sta	p3L
	lda	p2H
	ora	p3H
	sta	p3H
	jmp	push
.endproc


;********************************************************************************
;* NOT:
;*	Ones compliment
;********************************************************************************

.proc	notOP
	ldx	arithPtr

	lda	arithStack-1,x
	eor	#$FF
	sta	arithStack-1,x

	lda	arithStack-2,x
	eor	#$FF
	sta	arithStack-2,x

	rts
.endproc


