
;*********************************************************************************
;* relationals.s:
;*********************************************************************************

;********************************************************************************
;* This file is part of gibl:							*
;*	Gordons Interactive Basic Language                                      *
;********************************************************************************
;*    A Tiny Basic for the 6502 inspired by the NIBL Tiny Basic interpreter;	*
;*    "National Industrial Basic Language" originally for the INS8060 SC/MP	*
;*    system.									*
;*										*
;*    gibl is distributed under a "Source Available" license.			*
;*	It is NOT Open source and must not be treated as such.			*
;*										*
;*    See the file LICENSE for details.						*
;*										*
;*    gibl is Copyright (c) 2023 by Gordon Henderson				*
;********************************************************************************

	.macpack	generic

	.include	"data.h"
	.include	"ilExec.h"

	.include	"relationals.h"


; Some local flag bits..

fEQ	:=	$01
fLT	:=	$02
fGT	:=	$04


;********************************************************************************
;* Relational operators.
;*	EQ, NEQ, LSS, LEQ, GTR, GEQ
;********************************************************************************


;********************************************************************************
;* EQ:
;*	Equal to
;********************************************************************************

.proc	EQ
	jsr	multiCmp
	lda	num
	and	#fEQ
	bne	true			; Zero flag is set
	beq	false
.endproc


;********************************************************************************
;* NEQ:
;*	Not equal
;********************************************************************************

.proc	NEQ
	jsr	multiCmp
	lda	num
	and	#fEQ
	beq	true
;	bne	false
.endproc


;********************************************************************************
;* The return shenanigans.
;********************************************************************************

false:	lda	#0
	beq	pushResult

true:	lda	#1

pushResult:
	sta	arithStack-4,x		; low byte
	lda	#0
	sta	arithStack-3,x		; High

	dex
	dex
	stx	arithPtr

; Return via ilReturn

	jmp	ilReturn


;********************************************************************************
;* LSS:
;*	Less than
;********************************************************************************

.proc	LSS
	jsr	multiCmp
	lda	num
	and	#fLT | fEQ
	cmp	#fLT
	beq	true
	bne	false
.endproc


;********************************************************************************
;* LEQ:
;*	Less than or equal to.
;********************************************************************************

.proc	LEQ
	jsr	multiCmp
	lda	num
	and	#fLT | fEQ
	bne	true
	beq	false
.endproc


;********************************************************************************
;* GTR:
;*	Greater than
;********************************************************************************

.proc	GTR
	jsr	multiCmp
	lda	num
	and	#fLT | fEQ
	beq	true
	bne	false
.endproc


;********************************************************************************
;* GEQ:
;*	Greater than or equal
;********************************************************************************

.proc	GEQ
	jsr	multiCmp
	lda	num
	and	#fLT
	beq	true
	jmp	false
.endproc


;********************************************************************************
;* multiCmp:
;*	Routime to do all the tests at once and set bits in
;*	a register.
;********************************************************************************

.proc	multiCmp

; Pull numbers off the stack, store one in p2, the other in p3
;	Note that we leave the stack pointer in X for later...

	ldx	arithPtr

	lda	arithStack-1,x
	sta	p2H
	lda	arithStack-2,x
	sta	p2L
	lda	arithStack-3,x
	sta	p3H
	lda	arithStack-4,x
	sta	p3L

; Now, run a series of compares to set various flags...

	lda	#0
	sta	num

;  Start by subtracting

	sec
	lda	p3L
	sbc	p2L
	sta	lo
	lda	p3H
	sbc	p2H
	sta	hi

; Test for zero

	ora	lo
	bne	notZero
	lda	num
	ora	#fEQ
	sta	num
notZero:

; Test for <

	lda	hi
	bpl	notLt

; The result was negative, so 

	lda	num
	ora	#fLT
	sta	num
notLt:	
	rts
.endproc
